/**
* @author Dirk Nordmann
* @date   14.01.2002
*/
package img2wmf;

import util.*;
import java.io.*;
import java.awt.Image;

public class ImageToWMF extends ImageToImage
{
  private Image _image;
  private static int _counter = 0;
   	
  /**
  * Constructor
  */
  public ImageToWMF(String filename, int newWidth, int newHeight)
  {
    _image = new ImageLoader(filename).getImage();
    this.setWidth(newWidth);
    this.setHeight(newHeight);
    // Width & Height : wait until Java has loaded the image 
    while (this.getWidth() < 0 || this.getHeight() < 0)
    {
    	this.setWidth(_image.getWidth(null));
    	this.setHeight(_image.getHeight(null));
    }
  }

  /**
  * Returns the RTF Control-Word for this image.
  * @return String
  */
  public String getRtfControlWorld() 
  {
    return "wmetafile8";
  }

  /**
  * Returns the WMF data for this image.
  */
  public String getWmfData()
  {
    String s = "";
    try {
      ByteArrayOutputStream bytearrayoutputstream = new ByteArrayOutputStream();
      HexValuesOutputStream hexvaluesoutputstream = new HexValuesOutputStream(bytearrayoutputstream);
      if (this.convert(hexvaluesoutputstream)) {
     		s = bytearrayoutputstream.toString();
        bytearrayoutputstream.close();
 	    }
   	}
    catch(IOException ioexception) {
 	    System.out.println("*****[RtfImg.image2Wmf]: I/O-AUSNAHMEFEHLER -- while converting image to WMF : " + ioexception + "*****");
  	}
    return s;
  }
  
  /**
  * convert
  */  
  public boolean convert(OutputStream outputstream)
  {
      boolean flag = false;
      try
      {
  			//* Temporary Bitmap *
    		_counter++;
        File file = new File("~wmf" + _counter + ".bmp");
			
        DataOutputStream dataoutputstream = new DataOutputStream(new FileOutputStream(file.getPath()));
        flag = new ImageToBMP(_image).convert(dataoutputstream);
        dataoutputstream.close();
            
        if(flag)
        {
            BufferedInputStream bufferedinputstream = new BufferedInputStream(new FileInputStream(file.getPath()));
            flag = BMPToWMF(bufferedinputstream, file.length(), outputstream);
                bufferedinputstream.close();
            } else
            {
                System.err.println("[ImageToWMF]: ***** ERROR: Cannot convert image to BMP");
            }
			
			//* Remove temporary Bitmap from file-system *
        	file.delete();
        }
        catch(IOException ioexception)
        {
            System.err.println("[ImageToWMF]: ***** I/O ERROR: While converting image to WMF : " + ioexception);
        }
        return flag;
    }

    private boolean BMPToWMF(InputStream inputstream, long l, OutputStream outputstream)
    {
        try
        {
            int ai[] = {
                256, 2304, 3, 65535, 65535, 0, 65535, 65535, 0
            };
            int ai1[] = {
                1024, 0, 769, 2048
            };
            int ai2[] = {
                1280, 0, 2818, 0, 0
            };
            int ai3[] = {
                1280, 0, 3074, 65535, 65535
            };
            int ai4[] = {
                1024, 0, 1793, 1024
            };
            int ai5[] = {
                65535, 65535, 17167, 8192, 52224, 0, 65535, 65535, 0, 0, 
                65535, 65535, 0, 0
            };
            int ai6[] = {
                1024, 0, 1793, 0
            };
            int ai7[] = {
                768, 0, 0
            };
            int i = swap(this.getWidth());
            int j = swap(this.getHeight());
            long l1 = (l + 14L) / 2L;
            ai3[3] = j;
            ai3[4] = i;
            ai5[0] = hiBytes(swap(l1));
            ai5[1] = loBytes(swap(l1));
            ai5[6] = j;
            ai5[7] = i;
            ai5[10] = j;
            ai5[11] = i;
            ai[6] = ai5[0];
            ai[7] = ai5[1];
            long l2 = l1 + 34L;
            ai[3] = hiBytes(swap(l2));
            ai[4] = loBytes(swap(l2));
            
            //* WMF Header *
            writeIntArray(outputstream, ai);
            writeIntArray(outputstream, ai1);
            writeIntArray(outputstream, ai2);
            writeIntArray(outputstream, ai3);
            writeIntArray(outputstream, ai4);
            writeIntArray(outputstream, ai5);
            
            //* WMF Body (Bitmap-file content) *
            byte abyte0[] = new byte[4096];
            inputstream.read(abyte0, 0, 14);
            int k = 0;
            try
            {
                while((k = inputstream.read(abyte0)) != -1) 
                    outputstream.write(abyte0, 0, k);
            }
            catch (EOFException _ex) { }
            
            //* WMF Footer *
            writeIntArray(outputstream, ai6);
            writeIntArray(outputstream, ai7);
            
            return true;
        }
        catch(Exception exception)
        {
            System.out.println("[ImageToWMF]: ***** EXCEPTION: While converting BMP to WMF : " + exception);
            exception.printStackTrace();
            return false;
        }
    }

    private long swap(long l)
    {
        long l1 = (l & 0xffffffffff000000L) >> 24;
        l1 += (l & 0xff0000L) >> 8;
        l1 += (l & 65280L) << 8;
        l1 += (l & 255L) << 24;
        return l1;
    }

    private int swap(int i)
    {
        int j = (i & 0xff00) >> 8;
        j += (i & 0xff) << 8;
        return j;
    }

    private int hiBytes(long l)
    {
        return (int)((l & 0xffffffffffff0000L) >> 16);
    }

    private int loBytes(long l)
    {
        return (int)(l & 65535L);
    }

    private void writeIntArray(OutputStream outputstream, int ai[])
        throws IOException
    {
        for(int i = 0; i < ai.length; i++)
        {
            outputstream.write((ai[i] & 0xff00) >> 8);
            outputstream.write(ai[i] & 0xff);
        }

    }
}