/**
* @author Dirk Nordmann
* @date   14.01.2002
*/
package xml2rtf;

import java.io.*;
import java.util.*;
import util.*;

public class Batch
{

  private final static String ARG_DEBUG 	= "debug";
	  
  private static void usage()
  {
    System.err.println("Syntax:");
  	System.err.println("     java xml2rtf.Batch [-options] <HTML-File> [<RTF-File>]");
    System.err.println("mit folgenden Optionen:");
  	System.err.println("* Hilfe zur Syntax:                    -? -h -help");
    System.err.println("* Debug Informationen:                 -" + ARG_DEBUG );
  }

  private static String getStringArgValue(String arg, String argName )
  {
    return arg.substring( argName.length() + 1, arg.length() );
  }

  private static int getIntArgValue(String arg, String argName )
  {
    int rtn = -1;
    String argValue = getStringArgValue(arg, argName);
    try {
      rtn = Integer.parseInt( argValue );
    }
    catch (NumberFormatException ex) {
			System.out.println( "***** [Batch]: FEHLER in Argument '" + argName + "' -- '" + argValue + "' ist kein gltiger Integer-Wert! *****" );
			System.exit(1);
    }
    return rtn;
  }

  public static void main(String args[])
  {
    System.err.println("------------------------------------------");
  	System.err.println("ZMS HTML2RTF-Generator");
    System.err.println("Author: Dirk Nordmann (dnordmann@web.de)");
  	System.err.println("------------------------------------------");
   		
    //*****  get command-line arguments  *****
  	String html_file = null;
    String rtf_file = null;
  	for (int i = 0; i < args.length; i++)
    {
  		String arg = args[i];
    	if(arg.charAt(0) == '-')
      {
        //***** usage *****
  			if(arg.equals("-h") || 
    			arg.equals("-?") ||
      		arg.equals("-help") )
        {
  				usage();
    			System.exit(1);
      	}
  			//***** debug information *****
    		else if (arg.equals( "-" + ARG_DEBUG ))
      		Globals.setDebug(true);
  			//***** unknown argument: Usage *****
    		else {
      		usage();
  				System.exit(1);
    		}
      } 
  		else if(html_file == null)
   			html_file = arg;
      else if(rtf_file == null)
				rtf_file = arg;
  		else {
    		usage();
      	System.exit(1);
  		}
    }       	

  	//*****  check java version  *****
    Globals.writeDebug("[Batch]: Java version " + System.getProperty("java.version"));
		if (System.getProperty("java.version").compareTo("1.2")<0) {
      System.out.println("*****[Batch]: FEHLER -- Diese Anwendung bentigt Java 1.2 oder hher! *****");
      System.exit(1);
    }
        
    //*****  no input file given  *****
    if (html_file == null) {
  		System.out.println("*****[Batch]: FEHLER -- Keine Quelldatei angegeben! *****");
    	System.exit(1);
    }
    		
    //*****  input file must be HTML  *****
    else if (!html_file.toLowerCase().endsWith(".html")) {
      System.out.print("*****[Batch]: FEHLER -- Ungltige Quelldatei: '" + html_file + "'! ");
  		System.out.println("Nur .HTML Dateien werden untersttzt! *****");
    	System.exit(1);
    }
        
    //*****  default RTF filename identical with HTML filename  (modify extension)  *****
    if (rtf_file == null)
      rtf_file = FilePathUtility.SubstituteExtension(html_file, ".rtf");
    html_file = (new File(html_file)).getAbsolutePath();
    rtf_file = (new File(rtf_file)).getAbsolutePath();
            
    //*****  output file must be RTF  *****
    if(!rtf_file.toLowerCase().endsWith(".rtf")) {
  		System.out.print("*****[Batch]: FEHLER -- Ungltige Zieldatei: '" + rtf_file + "'! ");
    	System.out.println("Nur .RTF Dateien werden untersttzt! *****");
      System.exit(1);
    }
        
  	/* 	-----
        ----- perform conversion 
        -----
  	 */

    //*****  prepare conversion  *****
  	Globals.writeDebug("[Batch]: Konvertiere Datei '" + html_file + "' nach '" + rtf_file + "'...");
    long l = System.currentTimeMillis();
        
  	//*****  perform conversion  *****
    Converter.Convert(html_file,rtf_file);
        
  	//*****  finish conversion  *****
    Globals.writeDebug("[Batch]: Konvertierung abgeschlossen.");
  	System.out.print("*****FINISHED ");
    System.out.print( "in " + (double)(System.currentTimeMillis() - l) / 1000D);
  	System.out.println(" sec.");
        
    //*****  exit batch process  *****
  	System.exit(0);
  }

}