###################################################################################################
# _accessmanager.py
#
# Implementation of class AccessManager (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTML, HTMLFile
from AccessControl.User import UserFolder
import string
import urllib


###################################################################################################
###################################################################################################
###
###   C l a s s   A c c e s s M a n a g e r
###
###################################################################################################
###################################################################################################
class AccessManager: 

    # Management Interface to customize access.
    manage_users = HTMLFile('dtml/access/manage_users', globals())

    #-------------------------------------------------------------------------------------------
    #  AccessManager.noPermission:
    #-------------------------------------------------------------------------------------------
    viewPermission__roles__ = ['ZMSAdministrator','ZMSEditor','ZMSAuthor','ZMSSubscriber']
    def viewPermission(self): pass

    #-------------------------------------------------------------------------------------------
    #  AccessManager.initUserFolder:
    #-------------------------------------------------------------------------------------------
    def initUserFolder(self):
    
      # Add Roles.
      role_defs = {
               'ZMSAdministrator':{}
              ,'ZMSEditor':{'revoke':['ZMS Administrator']}
              ,'ZMSAuthor':{'revoke':['ZMS Administrator','ZMS Editor','Import/Export objects']}
              ,'ZMSSubscriber':{'grant':['Access contents information','View']}
              ,'Anonymous':{'grant':['Access contents information','View']}
              }
      for role in role_defs.keys():
        role_def = role_defs[role]
        # Add Local Role.
        if not role in self.valid_roles(): self._addRole(role)
        # Set permissions for Local Role.
        permissions = []
        for permission in self.permissionsOfRole('Manager'):
          if permission['selected']=='SELECTED':
            permissions.append(permission['name'])
        if role_def.has_key('revoke'):
          for revoke in role_def['revoke']:
            if revoke in permissions:
              permissions.remove(revoke)
        elif role_def.has_key('grant'):
          permissions = role_def['grant']
        self.manage_role(role_to_manage=role,permissions=permissions)
        
      # Clear acquired permissions.
      self.manage_acquiredPermissions([])


    #-------------------------------------------------------------------------------------------
    #  AccessManager.getUserFolder:
    #-------------------------------------------------------------------------------------------
    def getUserFolder(self):
      user_folder = None
      if len(self.objectValues(['User Folder'])) == 0:
        user_folder = UserFolder()
        self._setObject(user_folder.id, user_folder)
      user_folder = self.objectValues(['User Folder'])[0]
      return user_folder


    #-------------------------------------------------------------------------------------------
    # AccessManager.getUserPassword:
    # 
    # Returns password of user with specified ID.
    #-------------------------------------------------------------------------------------------
    def getUserPassword(self, name):
      user_folder = self.getUserFolder()
      user = user_folder.getUser(name)
      return user._getPassword()


    #-------------------------------------------------------------------------------------------
    # AccessManager.__del_user__:
    # 
    # Delete user with specified ID.
    #-------------------------------------------------------------------------------------------
    def __del_user__(self, name, lang, manage_lang, REQUEST):
    
        user_folder = self.getUserFolder()
        
        ##### Sender & Recipient ####
        mfrom = getattr(REQUEST['AUTHENTICATED_USER'],'email','info@zms-publishing.com')
        mto = getattr(user_folder.getUser(name),'email','')
        
        # Send notification to deleted user via MailHost.
        mailhost = self.getMailHost()
        if mailhost is not None and len(mto) > 0 and len(mfrom) > 0:
          print "[%s.__del_user__]: MailHost found..."%(self.meta_type)
          # Send message to deleted user.
          mbody \
              = '%s: %s (registration removed)'%(self.getLangStr('ATTR_USERID',manage_lang),name) \
              + '\n\n%s\n%s'%(self.getLangStr('WITH_BEST_REGARDS',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
          msubject = '%s (registration removed)'%self.getDocumentElement().aq_parent.title_or_id()
          print "[%s.__del_user__]: Send from %s to %s..."%(self.meta_type,mfrom,mto)
          mailhost.send(mbody,mto,mfrom,msubject)
          
        # Delete user physically.
        # -----------------------
        user_folder._delUsers([name])


    ###############################################################################################
    #	AccessManager.manage_editUsers:
    #
    #	Change or delete users.
    ###############################################################################################
    def manage_editUsers(self, lang, manage_lang, REQUEST):
      """ AccessManager.manage_editUsers """
      message = ''
      name = ''
      
      # Delete.
      # -------
      if REQUEST['btn'] == self.getLangStr('BTN_DELETE',manage_lang):
        if REQUEST.has_key('id'):
          name = REQUEST['id']
          self.__del_user__(name,lang,manage_lang,REQUEST)
          message = self.getLangStr('MSG_DELETED',manage_lang)%int(1)

      # Change.
      # -------
      elif REQUEST['btn'] == self.getLangStr('BTN_CHANGE',manage_lang):
        if REQUEST.has_key('id'):
          name = REQUEST['id']
          user_folder = self.getUserFolder()
          if user_folder is not None:
            user = user_folder.getUser(name)
            if user is not None:
              if REQUEST.has_key('email'):
                email = string.strip(REQUEST['email'])
                setattr(user,'email',email)
              if REQUEST.has_key('password') and \
                 REQUEST.has_key('confirm'):
                password = string.strip(REQUEST['password'])
                confirm = string.strip(REQUEST['confirm'])
                if len(password) >= 4 and password == confirm:
                  user.__ = password
              if REQUEST.has_key('langs'):
                langs = REQUEST['langs']
                if type(langs) != type([]):
                  langs = [langs]
                setattr(user,'langs',langs)
              if REQUEST.has_key('roles'):
                roles = REQUEST['roles']
                if type(roles) != type([]):
                  roles = [roles]
                setattr(user,'roles',roles)
              message = self.getLangStr('MSG_CHANGED',manage_lang)
          
      ###### PUBLIC ACCESS ####
      self.synchronizePublicAccess()
      
      # Return with message.                   
      return REQUEST.RESPONSE.redirect('manage_users?lang=%s&manage_lang=%s&manage_tabs_message=%s&id=%s'%(lang,manage_lang,urllib.quote(message),name))

        

    ###############################################################################################
    #	AccessManager.manage_addUser:
    #
    #	Add user.
    ###############################################################################################
    def manage_addUser(self, lang, manage_lang, REQUEST):
      """ AccessManager.manage_addUser """
      message = ''
      name = ''
        
      # Insert.
      # -------
      if REQUEST['btn'] == self.getLangStr('BTN_INSERT',manage_lang):
        user_folder = self.getUserFolder()
        if user_folder is not None:
            name = string.strip(REQUEST['_userid'])
            email = string.strip(REQUEST['_email'])
            password = string.strip(REQUEST['_password'])
            confirm = string.strip(REQUEST['_confirm'])
            langs = REQUEST['_langs']
            if type(langs) != type([]):
              langs = [langs]
            roles = REQUEST['_roles']
            if type(roles) != type([]):
              roles = [roles]
            domains = []
            
            if REQUEST['btn'] == self.getLangStr('BTN_INSERT',manage_lang):
                user_folder._addUser(name,password,confirm,roles,domains)
                user = user_folder.getUser(name)
                user.email = email
                user.langs = langs
                
                ##### Sender & Recipient ####
                mfrom = getattr(REQUEST['AUTHENTICATED_USER'],'email','info@zms-publishing.com')
                mto = email
                
                ##### Body ####
                mbody \
                  = '%s:\n%s'%(self.getDocumentElement().aq_parent.title_or_id(),self.absolute_url()) \
                  + '\n%s Management-Interface (registration):\n%s/manage'%(self.getDocumentElement().aq_parent.title_or_id(),self.absolute_url()) \
                  + '\n%s: %s'%(self.getLangStr('ATTR_USERID',manage_lang),name) \
                  + '\n%s: %s'%(self.getLangStr('ATTR_PASSWORD',manage_lang),password) \
                  + '\n%s: '%self.getLangStr('ATTR_ROLE',manage_lang)
                count = 0
                for role in roles:
                  if count > 0: mbody = mbody + ', '
                  mbody = mbody + self.getLangStr('ROLE_%s'%string.upper(role),manage_lang)
                  count = count + 1
                # With best regards.                
                mbody = mbody + '\n\n%s\n%s'%(self.getLangStr('WITH_BEST_REGARDS',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
                   
                ##### Subject ####
                msubject = '%s (registration)'%self.getDocumentElement().aq_parent.title_or_id()
                  
                print "[%s.manage_addUser]:"%(self.meta_type)
                print "From: %s"%mfrom
                print "To: %s"%mto
                print "Subject: %s"%msubject
                print "Body: %s"%mbody
                
                # Send notification to new user via MailHost.
                mailhost = self.getMailHost()
                if mailhost and len(mto) > 0 and len(mfrom) > 0:
                  print "[%s.manage_addUser]: MailHost found..."%(self.meta_type)
                  mailhost.send(mbody,mto,mfrom,msubject)
      
      ###### PUBLIC ACCESS ####
      self.synchronizePublicAccess()

      # Return with message.                   
      return REQUEST.RESPONSE.redirect('manage_users?lang=%s&manage_lang=%s&manage_tabs_message=%s&id=%s'%(lang,manage_lang,urllib.quote(message),name))


    #-------------------------------------------------------------------------------------------
    #  AccessManager.getUserFolders:
    #-------------------------------------------------------------------------------------------
    def _getUserFolders(self, obj, REQUEST):
        lst = []
        if obj:
          # Recurse up.
          oParent = obj.getParentNode()
          lst.extend(self._getUserFolders(oParent,REQUEST))
          # Scan User Folder.
          user_folder = obj.getUserFolder()
          if user_folder:
            lst.append(user_folder)
        return lst
        
    def getUserFolders(self, REQUEST):
        return self._getUserFolders(self,REQUEST)


    #-------------------------------------------------------------------------------------------
    #  AccessManager.hasAccess:
    #-------------------------------------------------------------------------------------------
    def hasAccess(self, REQUEST):
      return str(REQUEST['AUTHENTICATED_USER']) in self.getUserNames(REQUEST) or \
         REQUEST['AUTHENTICATED_USER'].has_role('Manager')


    #-------------------------------------------------------------------------------------------
    #  AccessManager.getUserNames:
    #-------------------------------------------------------------------------------------------
    def getUserNames(self, REQUEST):
      # Add users from ZMS tree.
      lst = []
      for user_folder in self.getUserFolders(REQUEST):
        lst.extend(user_folder.getUserNames())
      # Add users from Zope tree.
      try:
        oTree = self.getDocumentElement().aq_parent
        while oTree.id:
          if len(oTree.objectValues(['User Folder'])) > 0:
            user_folder = oTree.objectValues(['User Folder'])[0]
            lst.extend(user_folder.getUserNames())
          oTree = oTree.aq_parent
      except:
        pass
      # Return list of users names.
      return lst


    """
    ###############################################################################################
    #
    #  P U B L I C   A C C E S S 
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #	AccessManager.synchronizePublicAccess:
    #
    #	Grant / revoke public access.
    # ---------------------------------------------------------------------------------------------
    def synchronizePublicAccess(self):
      has_subscriber = 0
      user_folder = self.getUserFolder()
      for user in user_folder.getUsers():
         for role in user.getRoles():
           if role == 'ZMSSubscriber':
             has_subscriber = 1
      if has_subscriber:
        self.revokePublicAccess()
      else:
        self.grantPublicAccess()

    #-------------------------------------------------------------------------------------------
    #  AccessManager.grantPublicAccess:
    #-------------------------------------------------------------------------------------------
    def grantPublicAccess(self):
      print "[%s.grantPublicAccess]: "%self.meta_type
      if self.getLevel() > 0:
        permissions = []
        for perm in self.permissionsOfRole('Manager'):
          if perm['selected']=='SELECTED':
            permissions.append(perm['name'])
        self.manage_acquiredPermissions(permissions)
        for role in self.valid_roles():
	  self.manage_role(role_to_manage=role,permissions=[])
      self.manage_role(role_to_manage='Anonymous',permissions=['Access contents information','View'])
    

    #-------------------------------------------------------------------------------------------
    #  AccessManager.revokePublicAccess:
    #-------------------------------------------------------------------------------------------
    def revokePublicAccess(self):
      print "[%s.revokePublicAccess]: "%self.meta_type
      if self.getLevel() > 0:
        permissions = []
        for perm in self.permissionsOfRole('Manager'):
          if perm['selected']=='SELECTED':
            permissions.append(perm['name'])
        self.manage_acquiredPermissions([])
        for role in self.valid_roles():
	  self.manage_role(role_to_manage=role,permissions=permissions)
      self.manage_role(role_to_manage='Anonymous',permissions=['Access contents information'])

###################################################################################################
