###################################################################################################
# _metapatternmanager.py
#
# Implementation of class MetapatternManager (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from App.Common import package_home
from Globals import HTML, HTMLFile
import string
import urllib
import copy
import time
import tempfile
import os
# Product Imports.
import _globals
import _fileutil


###################################################################################################
###################################################################################################
###
###   class MetapatternObject
###
###################################################################################################
###################################################################################################
class MetapatternObject: 

    # Management Interface to add Meta-Objects.
    manage_addMetapatternForm = HTMLFile('dtml/metapattern/manage_addform', globals()) 
        
    ###############################################################################################
    # MetapatternObject.manage_addMetapattern:
    #
    # Wert fr Meta-Patterns hinzufgen.
    ###############################################################################################
    def manage_addMetapattern(self, btn, lang, manage_lang, REQUEST, RESPONSE):
        """ MetapatternObject.manage_addMetapattern """
        message = ""
        id = ""

        # Insert.
        # -------
        if btn == self.getLangStr('BTN_INSERT',manage_lang):

	  parent = self.getParentNode()
	  root = self.getDocumentElement()
	  folder = root.acl_metapatterns

	  ##### Register Meta-Pattern ####
	  id = REQUEST['id']
	  file = parent.manage_exportObject(id=self.id,download=1)
	  title = REQUEST['title']
	  folder.manage_addFile(id,file,title)
	  obj = getattr(folder,id)
	  obj.manage_addProperty('type',self.meta_type,'string')
	  obj.manage_addProperty('path',self.relative_obj_path(),'string')
	  
          # Return with message.
          message = urllib.quote(self.getLangStr('MSG_INSERTED',manage_lang)%title)
          return RESPONSE.redirect('%s/manage_customizeMetapatternForm?lang=%s&manage_lang=%s&manage_tabs_message=%s&id=%s'%(self.getDocumentElement().absolute_url(),lang,manage_lang,message,id))

        # Overwrite.
        # ----------
        if btn == self.getLangStr('BTN_OVERWRITE',manage_lang):

	  parent = self.getParentNode()
	  root = self.getDocumentElement()
	  folder = root.acl_metapatterns

	  ##### Overwrite Meta-Pattern ####
	  id = REQUEST['overwrite_id']
	  file = parent.manage_exportObject(id=self.id,download=1)
	  obj = getattr(folder,id)
	  obj.manage_upload(file)
	  obj.manage_editProperties({'path':self.relative_obj_path()})

          # Return with message.
          message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
          return RESPONSE.redirect('%s/manage_customizeMetapatternForm?lang=%s&manage_lang=%s&manage_tabs_message=%s&id=%s'%(self.getDocumentElement().absolute_url(),lang,manage_lang,message,id))

        # Return with message.
        return RESPONSE.redirect('../manage_main?lang=%s&manage_lang=%s#_%s'%(lang,manage_lang,self.id))
    
    ###############################################################################################
    # MetapatternObject.manage_instantiateMetapattern:
    #
    # Wert fr Meta-Patterns hinzufgen.
    ###############################################################################################
    def manage_instantiateMetapattern(self, lang, manage_lang, _sort_id, REQUEST, RESPONSE):
        """ MetapatternObject.manage_instantiateMetapattern """

        ob = None
        for id in self.getMetapatternIds(REQUEST):
          metapattern = self.getMetapattern(id,REQUEST)
          if metapattern['title'] == REQUEST['custom']:
            ob = metapattern
        
        # Create temp folder.
        folder_id = "~" + str(time.time())
        self.manage_addFolder(folder_id)
        folder = getattr(self,folder_id)
        
        # Import zexp to temp folder.
        path = INSTANCE_HOME + '/import/'
        filename = '%s.zexp'%ob['id']
        data = str(ob['ob'])
        print '[%s.manage_instantiateMetapattern]: file=%s%s (%i Bytes)'%(self.meta_type,path,filename,len(data))
        f = open(path+filename,'wb')
        f.write(data)
        f.close()
        _fileutil.importZexp(folder,path,filename)
        
        # Copy to current container with new id.
        obj = folder.objectValues()[0]
        old_id = obj.id
        new_id = self.getNewId()
        cb_copy_data = folder.manage_copyObjects(ids=[old_id])
        self.manage_pasteObjects(cb_copy_data)
        self.manage_renameObject(id='copy_of_%s'%old_id,new_id=new_id)
        
        obj = getattr(self,new_id)
        ##### Object State ####
        obj.setObjStateNew(REQUEST)
        ##### Init Properties ####
        obj_ver = obj.getObjVersion(REQUEST)
        obj_ver.sort_id = _globals.format_sort_id(_sort_id+1)
        for s_lang in self.getLanguages():
          obj.setObjProperty('change_uid',REQUEST['AUTHENTICATED_USER'].getId(),s_lang)
          obj.setObjProperty('change_dt',time.time(),s_lang)
        ##### VersionManager ####
        obj.__autoCommit__(REQUEST)
        
        ##### Normalize Sort-IDs ####
        self.normalizeSortIds()
        
        # Remove temp folder.
        self.manage_delObjects(ids=[folder_id],REQUEST=None)
        
        # Return with message.
        print '[%s.manage_instantiateMetapattern]: Return with message.'%(self.meta_type)
        message = urllib.quote(self.getLangStr('MSG_INSERTED',manage_lang)%ob['title'])
        return RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s#_%s'%(lang,manage_lang,message,new_id))


###################################################################################################
###################################################################################################
###
###   class MetapatternManager
###
###################################################################################################
###################################################################################################
class MetapatternManager(MetapatternObject): 

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = ()
    __editorPermissions__ = ()
    __administratorPermissions__ = (
		'manage_customizeMetapattern','manage_customizeMetapatternForm',
		)
    __ac_permissions__=(
		('ZMS Administrator', __administratorPermissions__),
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		)

    # ------------------------------------------------------------------------------------------
    # MetapatternManager.recurse_updateVersionMetaPatterns: 
    #
    # Update version.
    # ------------------------------------------------------------------------------------------
    def recurse_updateVersionMetaPatterns(self, lang, manage_lang, REQUEST):
      message = ""
      
      root = self.getDocumentElement()
      if not 'acl_metapatterns' in root.objectIds(['Folder']):
        root.manage_addFolder(id='acl_metapatterns',title='*** DO NOT DELETE OR MODIFY ***')
      
      return message

    # -------------------------------------------------------------------------------------------------
    #  initMetapatterns
    # -------------------------------------------------------------------------------------------------
    def initMetapatterns(self):
      root = self.getDocumentElement()
      folder = getattr(root,'acl_metapatterns')
      id = 'ZMSZubbForum'
      if not id in folder.objectIds(['File']):
        _fileutil.importZexp(folder,package_home(globals())+'/import/',id+'.zpat')
        #-- Reset path.
        ob = getattr(folder,id)
        ob.manage_changeProperties({'path':''})

    # ------------------------------------------------------------------------------------------
    # MetapatternManager.getMetapatternIds:
    # 
    # Returns list of IDs of Meta-Patterns.
    # ------------------------------------------------------------------------------------------
    def getMetapatternIds(self, REQUEST, meta_type=None):
      root = self.getDocumentElement()
      folder = root.acl_metapatterns
      ids = []
      for id in folder.objectIds(['File']):
        metapattern = self.getMetapattern(id,REQUEST)
        if meta_type is None or metapattern['type']==meta_type:
          ids.append(metapattern['id'])
      return ids

    # ------------------------------------------------------------------------------------------
    # MetapatternManager.getMetapattern:
    # 
    # Returns Meta-Pattern identified by ID.
    # ------------------------------------------------------------------------------------------
    def getMetapattern(self, id, REQUEST):
      root = self.getDocumentElement()
      folder = root.acl_metapatterns
      ob = getattr(folder,id)
      ob_title = ob.title_or_id()
      ob_type = ob.getProperty('type')
      ob_path = ob.getProperty('path')
      ob_src = None
      if ob_path:
        ob_src = ob.getLinkObj('{$' + ob_path + '}',REQUEST)
      return {'id':id,'title':ob_title,'type':ob_type,'path':ob_path,'src':ob_src,'ob':ob}

    # ------------------------------------------------------------------------------------------
    # MetapatternManager.delMetapattern:
    # 
    # Delete Meta-Pattern specified by ID.
    # ------------------------------------------------------------------------------------------
    def delMetapattern(self, id):
      root = self.getDocumentElement()
      folder = root.acl_metapatterns
      folder.manage_delObjects([id],None)

    # Management Interface to customize Meta-Patterns.
    manage_customizeMetapatternForm = HTMLFile('dtml/metapattern/manage_customizeform', globals()) 

    ###############################################################################################
    # MetapatternManager.manage_customizeMetapattern:
    #
    # Werte fr Meta-Patterns ndern.
    ###############################################################################################
    def manage_customizeMetapattern(self, btn, lang, manage_lang, REQUEST, RESPONSE):
        """ MetapatternManager.manage_customizeMetapattern """
        message = ''
        id = ''
        
        # Delete.
        # -------
        if btn == self.getLangStr('BTN_DELETE',manage_lang):
          id = REQUEST['id']
          self.delMetapattern(id)
          id = ''
          message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))        
        
        # Change.
        # -------
        elif btn == self.getLangStr('BTN_CHANGE',manage_lang):
          id = REQUEST['id']
          new_id = string.strip(REQUEST['new_id'])
          new_title = string.strip(REQUEST['new_title'])
          root = self.getDocumentElement()
          folder = root.acl_metapatterns
          if id != new_id:
            folder.manage_renameObject(id=id,new_id=new_id)
          obj = getattr(folder,new_id)
          obj.title = new_title
          id = new_id
          message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))        
        
        # Export.
        # -------
        elif btn == self.getLangStr('BTN_EXPORT',manage_lang):
          id = REQUEST['id']
          root = self.getDocumentElement()
	  folder = root.acl_metapatterns
          content_type = 'application/zip'
          export = folder.manage_exportObject(id=id,download=1)
          filename = id + '.zpat'
          RESPONSE.setHeader('Content-Type',content_type)
          RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
          return export
        
        # Import.
        # -------
        elif btn == self.getLangStr('BTN_IMPORT',manage_lang):
          root = self.getDocumentElement()
	  folder = root.acl_metapatterns
          idsBefore = self.getMetapatternIds(REQUEST)
          #-- Create temporary folder.
          path = tempfile.mktemp() + os.sep
          filename = 'metapattern.zexp'
          #-- Save file to temporary folder.
	  _fileutil.exportObj(REQUEST['file'],'%s/%s'%(path,filename))
	  #-- Import zexp
	  _fileutil.importZexp(folder,path,filename)
          #-- Remove temporary folder.
          _fileutil.rmFilePath(path)
          #-- Retrieve id.
          idsAfter = self.getMetapatternIds(REQUEST)
          for id in idsAfter:
            if id not in idsBefore:
              new_id = id
          id = new_id
          #-- Reset path.
          ob = getattr(folder,id)
          ob.manage_changeProperties({'path':''})

        # Init.
        # -----
        elif btn == self.getLangStr('BTN_INIT',manage_lang):
          self.initMetapatterns()
          id = ''
        
        # Return with message.
        return RESPONSE.redirect('manage_customizeMetapatternForm?lang=%s&manage_lang=%s&manage_tabs_message=%s&id=%s'%(lang,manage_lang,message,id))

###################################################################################################
