###################################################################################################
# zms.py
#
# Implementation of class ZMS (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
import Products
from Globals import ImageFile, HTMLFile, MessageDialog, Persistent
from AccessControl.User import UserFolder
from App.Common import package_home
from OFS.Image import Image
from sys import *
import DateTime
import string 
import urllib
import time
# Product imports.
import _builder
import _cachemanager
import _multilangmanager
import _ftpmanager
import _metadictmanager
import _metaobjmanager
import _metapatternmanager
import _globals
import _fileutil
import _textformatmanager
import _versionmanager
import _zcatalogmanager
import _printablemanager
from zmscontainerobject import ZMSContainerObject
from zmsrubrik import ZMSRubrik
from zmsdocument import ZMSDocument
from zmstextarea import ZMSTextarea
from zmsgraphic import ZMSGraphic
from zmstable import ZMSTable
from zmsfile import ZMSFile
from zmsnote import ZMSNote
from zmsteasercontainer import ZMSTeaserContainer
from zmsteaserelement import ZMSTeaserElement
from zmscustom import ZMSCustom
from zmssysfolder import ZMSSysFolder
from zmslinkcontainer import ZMSLinkContainer


###################################################################################################
###################################################################################################
###   
###   C o m m o n   F u n c t i o n ( s )
###   
###################################################################################################
###################################################################################################

# -------------------------------------------------------------------------------------------------
#  _getNewId
# -------------------------------------------------------------------------------------------------
def _getNewId(_obj, _id):
  for obj in _obj.objectValues(_obj.dGlobalAttrs.keys()):
    try:
      sObjId = str(obj.id)
      iObjId = string.atoi(sObjId[1:])
      if iObjId > _id:
        _id = iObjId
      newid = _getNewId(obj,_id)
      if newid > _id:
        _id = newid
    except:
      pass
  return _id

# -------------------------------------------------------------------------------------------------
#  createForum
# -------------------------------------------------------------------------------------------------
def createForum(self, REQUEST):
  print "[createForum]"
  lang = REQUEST['lang']
  
  ##### Create ####
  obj = ZMSSysFolder(self.getNewId(),99999)
  self._setObject(obj.id, obj)
  
  for oItem in self.objectValues():
    if oItem.id == obj.id:
      ##### Object State ####
      oItem.setObjStateNew(REQUEST)
      ##### Init Properties ####
      oItem.setObjProperty('active',1,lang)
      oItem.setObjProperty('title','Forum',lang)
      oItem.setObjProperty('titleshort','Forum',lang)
      ##### ZUBB ####
      _fileutil.importZexp(oItem,package_home(globals())+'/import/','forum.zexp')
      ##### index_html ####
      dtml_id = 'index_html'
      dtml_title = 'Redirect to forum'
      dtml_raw = ""
      dtml_raw = dtml_raw + '<!-- BO %s -->\n'%dtml_id
      dtml_raw = dtml_raw + '  <dtml-call "RESPONSE.redirect(forum.absolute_url())">\n'
      dtml_raw = dtml_raw + '<!-- EO %s -->\n'%dtml_id
      oItem.manage_addDTMLMethod(dtml_id,dtml_title,dtml_raw)
      ##### VersionManager ####
      oItem.__autoCommit__(REQUEST)
  
  ##### Normalize Sort-IDs ####
  self.normalizeSortIds()


###################################################################################################
###################################################################################################
###   
###   C o n s t r u c t o r ( s )
###   
###################################################################################################
###################################################################################################

manage_addZMSForm = HTMLFile('manage_addzmsform', globals()) 
def manage_addZMS(self, id, lang, manage_lang, lang_label, REQUEST, RESPONSE):
  """ manage_addZMS """
  
  if REQUEST['btn']=='Add':
    
    root = self
    
    # Add TemplateSet.
    # ----------------
    if REQUEST.has_key('theme') and len(REQUEST['theme']) > 0:
      ##### Theme ####
      theme = REQUEST['theme']
      theme_zexp = theme + '.zexp'
      _fileutil.importZexp(self,package_home(globals())+'/import/',theme_zexp)
      ##### Folder-Id ####
      folder_id = REQUEST['folder_id']
      self.manage_renameObject(theme,folder_id)
      ##### Manager ####
      root = getattr(self,folder_id)
      user_folder = getattr(root,'acl_users',None)
      if user_folder is not None:
        name = REQUEST['manager_name']
        password = REQUEST['manager_password']
        confirm = REQUEST['manager_confirm']
        roles = ['Manager']
        domains = []
        user_folder._addUser(name,password,confirm,roles,domains)
    
    # Add ZMS-Object.
    # ---------------
    obj = ZMS(id)
    root._setObject(obj.id, obj)
    
    for oItem in root.objectValues():
      if oItem.id == obj.id:
        
        ##### WWW-Site ####
        oItem.attr_provider_ftp_site = REQUEST['asp_ip_or_domain']
        oItem.asp_ip_or_domain = REQUEST['asp_ip_or_domain']
        oItem.asp_uid = REQUEST['asp_uid']
        oItem.asp_pwd = ''
        if len(REQUEST['asp_pwd']) > 0 and REQUEST['asp_pwd'] == REQUEST['asp_pwd_confirm']:
          oItem.asp_pwd = REQUEST['asp_pwd']
        
        ##### Add UserFolder ####
        user_folder = UserFolder()
        oItem._setObject(user_folder.id, user_folder)
        oItem.initUserFolder()
        
        ##### Add ZCatalog ####
        oItem.setCatalogPeriod()
        oItem.recreateCatalog(REQUEST)
        
        ##### Properties ####
        oItem.autocommit = 1
        oItem.setPrimaryLanguage(lang)
        oItem.__set_lang__(lang,lang_label,'',manage_lang)
        
        ##### Object State ####
        oItem.setObjStateNew(REQUEST)
        ##### Init Properties ####
        oItem.setObjProperty('active',1,lang)
        oItem.setObjProperty('attr_active_start',time.time(),lang)
        oItem.setObjProperty('titleshort','ZMS home',lang)
        oItem.setObjProperty('title','ZMS - ZOPE-based contentmanagement system for science, technology and medicine',lang)
        oItem.setObjProperty('change_uid',REQUEST['AUTHENTICATED_USER'],lang)
        oItem.setObjProperty('change_dt',time.time(),lang)
        ##### VersionManager ####
        oItem.__autoCommit__(REQUEST)
        
        ##### Meta-Dictionaries ####
        oItem.__init_metadicts__()
        _metadictmanager.initMetaDictAttrs(oItem)
        
        ##### Version ####
        oItem.updateVersion(lang,manage_lang,REQUEST)
        
        ##### Forum ####
        if REQUEST.has_key('forum') and len(REQUEST['forum']) > 0:
          createForum(oItem,REQUEST)
    
    # Return with message.
    message = obj.getLangStr('MSG_INSERTED',manage_lang)%obj.meta_type
    RESPONSE.redirect('%s/%s/manage?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(root.absolute_url(),obj.id,lang,manage_lang,urllib.quote(message)))
  
  else:
    RESPONSE.redirect('%s/manage_main'%self.absolute_url())

###################################################################################################
###################################################################################################
###   
###   C l a s s 
###   
###################################################################################################
###################################################################################################
class ZMS(
	ZMSContainerObject,
	_printablemanager.PrintableManager,
	_builder.Builder,
	_ftpmanager.FtpManager,
	_multilangmanager.MultiLanguageManager,
	_metadictmanager.MetadictManager,
	_metaobjmanager.MetaobjManager,
	_metapatternmanager.MetapatternManager,
	_cachemanager.CacheManager,
	_textformatmanager.TextFormatManager,
	_zcatalogmanager.ZCatalogManager
	):

    # Version-Info.
    # ------------
    zms_version = '1.0'
    zms_build = '086'
    zms_patch = 'd'

    # Properties.
    # -----------
    meta_type = 'ZMS'
    icon = "misc_/zms/zms.gif"
    icon_disabled = 'misc_/zms/zms_disabled.gif'

    # Management Options.
    # -------------------
    manage_options = (
	{'label': 'TAB_EDIT',         'action': 'manage_main'},
	{'label': 'TAB_PROPERTIES',   'action': 'manage_properties'},
	{'label': 'TAB_ACCESS',       'action': 'manage_users'},
	{'label': 'TAB_IMPORTEXPORT', 'action': 'manage_importexport'},
	{'label': 'TAB_TASKS',        'action': 'manage_tasks'},
	{'label': 'TAB_CONFIGURATION','action': 'manage_customizeForm'},
	{'label': 'TAB_PREVIEW',      'action': 'preview_html'}, # empty string defaults to index_html
	)

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = (
		'manage','manage_main','manage_menu',
		'manage_addZMSRubrik','manage_addZMSDocument','manage_addZMSTextarea','manage_addZMSGraphic','manage_addZMSTable','manage_addZMSFile','manage_addZMSTeaserContainer','manage_addZMSNote','manage_addZMSLinkContainer',
		'manage_deleteObjs','manage_moveObjUp','manage_moveObjDown','manage_cutObjects','manage_copyObjects','manage_pasteObjs',
		'manage_properties','manage_changeProperties',
		'manage_sitemap','manage_search','manage_tasks',
		'manage_rollbackVersionChanges','manage_requestVersionChanges',
		'manage_users',
		)
    __editorPermissions__ = (
		'manage_commitVersionChanges',
		)
    __administratorPermissions__ = (
		'manage_addZMSSysFolder',
		'manage_customizeForm','manage_customize','manage_updateCatalog',
		'manage_customizeDesignForm','manage_customizeDesign',
		)
    __ac_permissions__=(
		('ZMS Administrator', __administratorPermissions__),
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		('Import/Export objects', ('manage_importexport','manage_import','manage_export',) ),
		# ('Manage users', ('manage_users',) ),
		)

    # Properties.
    # -----------
    __obj_attrs__ = {
        'change_uid':{'datatype':'LangString','lang_inherit':-1},
        'change_dt':{'datatype':'LangDate','lang_inherit':-1},
        'work_uid':{'datatype':'LangString','lang_inherit':-1},
        'work_dt':{'datatype':'LangDate','lang_inherit':-1},
        'active':{'datatype':'LangInt','type':'checkbox','lang_inherit':0},
        'attr_active_start':{'datatype':'LangDate','lang_inherit':0},
        'attr_active_end':{'datatype':'LangDate','lang_inherit':0},
        'title':{'datatype':'LangString','size':40},
        'titleshort':{'datatype':'LangString','size':20},
        'titleimage':{'datatype':'LangMyImage'},
        'levelnfc':{'datatype':'string'},
        'attr_cacheable':{'datatype':'boolean','default':1},
        # Meta-Data
        'attr_dc_coverage':{'datatype':'string','default':'obligation'},
        # Meta-Dictionaries        
        '$metadict':{'datatype':'MetaDict'},
    }
    __extra_obj_attrs__ = {
        # Global-Languages
        'primary_language':{'datatype':'string'},
        'attr_languages':{'datatype':'dictionary'},
        '__attr_dc_metaobjs__':{'datatype':'dictionary'},
        '__attr_dc_metadicts__':{'datatype':'list'},
        '__attr_text_formats__':{'datatype':'dictionary'},
        '__attr_text_formats_default__':{'datatype':'string'},
    }

    # Globals.
    # --------
    dGlobalAttrs = {
	'ZMS':{'obj_class':None,'meta_dict':1,'icon':icon,'constructor':None,'obj_attrs':__obj_attrs__},
	'ZMSRubrik':{'obj_class':ZMSRubrik,'meta_dict':1,'icon':ZMSRubrik.icon,'constructor':'manage_addzmsrubrikform','obj_attrs':ZMSRubrik.__obj_attrs__},
	'ZMSDocument':{'obj_class':ZMSDocument,'meta_dict':1,'icon':ZMSDocument.icon,'constructor':'manage_addzmsdocumentform','obj_attrs':ZMSDocument.__obj_attrs__},
	'ZMSTextarea':{'obj_class':ZMSTextarea,'meta_dict':1,'icon':ZMSTextarea.icon,'constructor':'manage_addZMSTextarea','obj_attrs':ZMSTextarea.__obj_attrs__},
	'ZMSGraphic':{'obj_class':ZMSGraphic,'meta_dict':1,'icon':ZMSGraphic.icon,'constructor':'manage_addZMSGraphic','obj_attrs':ZMSGraphic.__obj_attrs__},
	'ZMSTable':{'obj_class':ZMSTable,'meta_dict':0,'icon':ZMSTable.icon,'constructor':'manage_addzmstableform','obj_attrs':ZMSTable.__obj_attrs__},
	'ZMSFile':{'obj_class':ZMSFile,'meta_dict':1,'icon':ZMSFile.icon,'constructor':'manage_addzmsfileform','obj_attrs':ZMSFile.__obj_attrs__},
	'ZMSNote':{'obj_class':ZMSNote,'meta_dict':0,'icon':ZMSNote.icon,'constructor':'manage_addZMSNote','obj_attrs':ZMSNote.__obj_attrs__},
	'ZMSTeaserElement':{'obj_class':ZMSTeaserElement,'meta_dict':0,'icon':ZMSTeaserElement.icon,'constructor':'manage_addZMSTeaserElement','obj_attrs':ZMSTeaserElement.__obj_attrs__},
	'ZMSTeaserContainer':{'obj_class':ZMSTeaserContainer,'meta_dict':0,'icon':ZMSTeaserContainer.icon,'constructor':'manage_addZMSTeaserContainer','obj_attrs':ZMSTeaserContainer.__obj_attrs__},
	'ZMSCustom':{'obj_class':ZMSCustom,'meta_dict':0,'icon':ZMSCustom.icon,'constructor':None,'obj_attrs':ZMSCustom.__obj_attrs__},
	'ZMSSysFolder':{'obj_class':ZMSSysFolder,'meta_dict':1,'icon':ZMSSysFolder.icon,'constructor':'manage_addzmssysfolderform','obj_attrs':ZMSSysFolder.__obj_attrs__},
	'ZMSLinkContainer':{'obj_class':ZMSLinkContainer,'meta_dict':0,'icon':ZMSLinkContainer.icon,'constructor':'manage_addZMSLinkContainer','obj_attrs':ZMSLinkContainer.__obj_attrs__},
	}

    # Register DTML-Templates.
    # ------------------------
    index_html = HTMLFile('dtml/zms/index', globals()) # index_html
    inactive_html = HTMLFile('dtml/zms/inactive', globals()) # inactive_html
    f_bodyContent = HTMLFile('dtml/zms/f_bodycontent', globals()) # Template: Body-Content / Element
    f_bodyContent_Float = HTMLFile('dtml/zms/f_bodycontent_float', globals()) # Template: Body-Content / Textkrper
    f_bodyContent_Sitemap = HTMLFile('dtml/zms/f_bodycontent_sitemap', globals()) # Template: Sitemap
    f_bodyContent_Search = HTMLFile('dtml/zms/f_bodycontent_search', globals()) # Template: Search
    search_nav_html = HTMLFile('dtml/zms/search_nav', globals()) # search_nav_html
    headScript = HTMLFile('dtml/zms/headscript', globals()) # JavaScript
    headMeta = HTMLFile('dtml/zms/headmeta', globals()) # Metatags
    headMeta_DC = HTMLFile('dtml/zms/headmeta_dc', globals()) # Dublin-Core Metatags
    headCStyleSheet = HTMLFile('dtml/zms/headcstylesheet', globals()) # Template_L1: CSS-Referenz
    f_sitemap = HTMLFile('dtml/zms/f_sitemap', globals()) # f_sitemap
    f_standard_html_request = HTMLFile('dtml/zms/f_standard_html_request', globals()) # f_standard_html_request
    f_standard_html_header = HTMLFile('dtml/zms/f_standard_html_header', globals()) # f_standard_html_header
    f_standard_html_footer = HTMLFile('dtml/zms/f_standard_html_footer', globals()) # f_standard_html_footer


    """
    ###############################################################################################    
    #
    #   CONSTRUCTOR
    #
    ###############################################################################################    
    """
    
    ###############################################################################################
    # ZMS.__init__: 
    #
    # Constructor (initialise a new instance of ZMS).
    ###############################################################################################
    def __init__(self, id): 
      """ ZMS.__init__ """
      self.id = id
      self.logo = ImageFile('Products/zms/images/logo.gif')
      self.stylesheet = HTMLFile('dtml/zms/__init__stylesheet', globals())(self)


    # ---------------------------------------------------------------------------------------------
    #	ZMS.Logo
    # ---------------------------------------------------------------------------------------------
    def Logo(self, REQUEST, RESPONSE): 
      """ZMS.Logo"""
      return self.logo.index_html(REQUEST,RESPONSE)

    # ---------------------------------------------------------------------------------------------
    #	ZMS.getDocumentElement:
    #
    #	The root element of the site.
    # ---------------------------------------------------------------------------------------------
    def getDocumentElement(self):
      return self

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getNewId:
    #
    #  Returns new (unique) object-id.
    # ---------------------------------------------------------------------------------------------
    def getNewId(self):
      rtn = _getNewId(self,0) + 1
      return 'e%i'%rtn

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getDCCoverage
    # ---------------------------------------------------------------------------------------------
    def getDCCoverage(self, REQUEST): return 'obligation'

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getMailHost:
    #
    #  Returns MailHost-object (None if there is no MailHost).
    # ---------------------------------------------------------------------------------------------
    def getMailHost(self):
      mailhost = None
      root = self.aq_parent
      if len(root.objectValues(['Mail Host'])) == 1:
        mailhost = root.objectValues(['Mail Host'])[0]
      elif getattr(root,'MailHost',None) is not None:
        mailhost = getattr(root,'MailHost',None)
      return mailhost


    """
    ###############################################################################################    
    #
    #   ZMS - Autocommit
    #
    ###############################################################################################    
    """

    # ---------------------------------------------------------------------------------------------
    #	ZMS.getAutocommit
    # ---------------------------------------------------------------------------------------------
    def getAutocommit(self): 
      return getattr(self,'autocommit',0)

    # ---------------------------------------------------------------------------------------------
    #	ZMS.doAutocommit
    # ---------------------------------------------------------------------------------------------
    def doAutocommit(self): 
      _versionmanager.doAutocommit(self)


    ###############################################################################################
    # ZMS.manage_customizeAsp: 
    #
    # Change ASP properties.
    ###############################################################################################
    def manage_customizeAsp(self, lang, manage_lang, REQUEST, RESPONSE): 
      """ ZMS.manage_customizeAsp """
      
      self.asp_ip_or_domain = REQUEST['asp_ip_or_domain']
      if REQUEST.has_key('asp_uid'): 
        self.asp_uid = REQUEST['asp_uid']
      if REQUEST.has_key('asp_pwd') and \
         REQUEST.has_key('asp_pwd_confirm'):
        if len(REQUEST['asp_pwd']) > 0 and REQUEST['asp_pwd'] == REQUEST['asp_pwd_confirm']:
          self.asp_pwd = REQUEST['asp_pwd']
      
      # Return with message.
      message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
      return RESPONSE.redirect('manage_customizeForm?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))


    ###############################################################################################
    # ZMS.manage_customizeAutoPack: 
    #
    # Change Auto-Pack properties.
    ###############################################################################################
    def manage_customizeAutoPack(self, btn, lang, manage_lang, REQUEST, RESPONSE): 
      """ ZMS.manage_customizeAutoPack """

      if btn == 'Change':
        try:      
          self.property_threshold = int(REQUEST['property_threshold'])
          self.property_days = int(REQUEST['property_days'])
        except:
          self.property_threshold = None
          self.property_days = None
          
      elif btn == 'Clear':
        self.property_threshold = None
        self.property_days = None
      
      # Return with message.
      message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
      return RESPONSE.redirect('manage_customizeForm?lang=%s&manage_lang=%s&manage_tabs_message=%s#_system'%(lang,manage_lang,message))


    ###############################################################################################
    # ZMS.manage_customize: 
    #
    # Change system properties.
    ###############################################################################################
    def manage_customize(self, lang, manage_lang, REQUEST, RESPONSE): 
      """ ZMS.manage_customize """
      
      ##### Apply Auto-Commit ####
      old_autocommit = self.getAutocommit()
      if REQUEST.has_key('autocommit'):
        self.autocommit = int(REQUEST['autocommit'])
      else:
        self.autocommit = 0
      
      ##### Activate Auto-Commit and commit all pending changes? ####
      new_autocommit = self.getAutocommit()
      if old_autocommit == 0 and \
         new_autocommit == 1:
        self.doAutocommit()
      
      # Return with message.
      message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
      return RESPONSE.redirect('%s/manage_customizeForm?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(self.absolute_url(),lang,manage_lang,message))


    """
    ###############################################################################################    
    #
    #   ZMS - Versions
    #
    ###############################################################################################    
    """

    # ---------------------------------------------------------------------------------------------
    #	ZMS.updateVersion:
    #
    #	Update version.
    # ---------------------------------------------------------------------------------------------
    def updateVersion(self, lang, manage_lang, REQUEST):
      #++ print "[%s.updateVersion]"%(self.meta_type)
      message = ''
      build = getattr(self,'build','000')
      if build < self.zms_build:
        self.initUserFolder()
        self.recurse_updateVersionMetaDicts(lang,manage_lang,REQUEST) # Must be executed first!
        self.recurse_updateVersionMetaObjs(lang,manage_lang,REQUEST)
        self.recurse_updateVersionMetaPatterns(lang,manage_lang,REQUEST)
        self.recurse_updateVersionLanguages(lang,manage_lang,REQUEST)
        message = self.recurse_updateVersion(lang,manage_lang,REQUEST)
        message = message + 'Updated object-attributes from Build #%s to newer Build #%s!'%(build,self.zms_build)
        setattr(self,'build',self.zms_build)
      message = message + self.autoPackVersion(lang,manage_lang,REQUEST)
      return message

    # ---------------------------------------------------------------------------------------------
    #	ZMS.autoPackVersion:
    #
    #	Auto-Pack version.
    # ---------------------------------------------------------------------------------------------
    def autoPackVersion(self, lang, manage_lang, REQUEST):
      message = ''
      threshold = getattr(self,'property_threshold',None)
      days = getattr(self,'property_days',None)
      if threshold is not None and days is not None:
        undoable_transactions = self.aq_parent.undoable_transactions(first_transaction=None,last_transaction=None,PrincipiaUndoBatchSize=999)
        transactions = len(undoable_transactions)
        #++ print "[%s.autoPackVersion]: Transactions: %i (>=Threshold: %i?)"%(self.meta_type,transactions,threshold)
        if transactions >= threshold:
	  t0 = undoable_transactions[-1]['time'].timeTime()+days*86400.0
	  t1 = time.time()
	  #++ print "[%s.autoPackVersion]: Next Scheduled Time: %s"%(self.meta_type,self.getLangFmtDate(t0,manage_lang))
	  #++ print "[%s.autoPackVersion]: Current Time: %s"%(self.meta_type,self.getLangFmtDate(t1,manage_lang))
          if t1 >= t0:
            #++ print "[%s.autoPackVersion]: Remove previous revisions of objects that are older than %i days."%(self.meta_type,days)
            try:
              self.Control_Panel.Database.manage_pack(days)
            except:
              pass
      return message


    """
    ###############################################################################################    
    #
    #   Customize
    #
    ###############################################################################################    
    """

    # Customize Interfaces.
    manage_customizeForm = HTMLFile('dtml/zms/manage_customizeform', globals()) 
    manage_customizeDesignForm = HTMLFile('dtml/zms/manage_customizedesignform', globals()) 
    manage_customizeProviderForm = HTMLFile('dtml/zms/manage_customizeproviderform', globals()) 

    ###############################################################################################
    # ZMS.manage_customizeDesign: 
    #
    # Change layout (CSS Stylesheet and Logo).
    ###############################################################################################
    def manage_customizeDesign(self, lang, manage_lang, REQUEST, RESPONSE):
      """ ZMS.manage_customizeDesign """
      
      # Change Stylesheet / Sitemap.
      if REQUEST['btn'] == self.getLangStr('BTN_CHANGE',manage_lang):
        if REQUEST.has_key('stylesheet'):
          self.stylesheet = REQUEST['stylesheet']
        if REQUEST.has_key('attr_layoutsitemap'):
          if len(REQUEST['attr_layoutsitemap'])>0:
            self.attr_layoutsitemap = int(REQUEST['attr_layoutsitemap'])
          elif hasattr(self,'attr_layoutsitemap'):
            delattr(self,'attr_layoutsitemap')
      
      # Upload Logo.
      elif REQUEST['btn'] == self.getLangStr('BTN_UPLOAD',manage_lang):
        self.logo = Image(id='logo',title='',file='')
        self.logo.manage_upload(REQUEST['file'],REQUEST)
      elif REQUEST['btn'] == self.getLangStr('BTN_CLEAR',manage_lang):
        self.logo = None
      
      # Return with message.
      message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
      return RESPONSE.redirect('manage_customizeDesignForm?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))


    """
    ###############################################################################################
    ###  
    ###  DOM-Methoden 
    ### 
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #	ZMS.getParentNode:
    #
    #	The parent of this node. 
    #	All nodes except root may have a parent.
    # ---------------------------------------------------------------------------------------------
    def getParentNode(self): 
      return None


    """
    ###############################################################################################    
    #
    #   BUILDER
    #
    ###############################################################################################    
    """

    ###############################################################################################
    # ZMS.xmlOnStartElement(self, sTagName, dAttrs, oParentNode, oRoot):
    #
    # handler for XML-Builder (_builder.py)
    ###############################################################################################
    def xmlOnStartElement(self, sTagName, dTagAttrs, oParentNode, oRoot):
      """ZMS.xmlOnStartElement"""
      _globals.myDebugger.write("[%s.xmlOnStartElement]: sTagName=%s"%(self.meta_type,sTagName))
      
      self.dTagStack = _globals.MyStack()
      self.dValueStack  = _globals.MyStack()
      
      # WORKAROUND! The member variable "aq_parent" does not contain the right parent object at 
      # this stage of the creation process (it will later on!). Therefore, we introduce a special
      # attribute containing the parent object, which will be used by xmlGetParent() (see below).
      self.oParent = None

###################################################################################################
